#!/bin/sh

PROG=$(basename $0 .sh)

# gpg timestamp
LC_TIME=en_US.UTF-8
export LC_TIME

DEFAULT_MACHINE=dss11-1gb-t1

if [ ! -f run_tests.sh ]; then
    echo "Invalid directory. Please run as ./run_tests.sh from the test/ directory."
    exit 1
fi

. ../src/parse_config_file.sh

usage() {
    echo "Usage:"
    echo "  $PROG [options] [test_directory]..."
    echo "Options:"
    echo "  -h"
    echo "     Show this help message."
    echo "  -d"
    echo "     Dry-run mode. Perform checks on the downloaded image, but do not reflash"
    echo "  -v"
    echo "     Verbose mode"
    echo "  -k"
    echo "     do not stop on failure"
    echo "  -l"
    echo "     list available test cases"
    echo "  -m"
    echo "     only run tests for machine, e.g. (dss11e/dss11-1gb-t1)"
    echo "  -u"
    echo "     Update the test expected output with the new output in case of failure"
    echo "  -s"
    echo "     Only create sysroot then stop test, expects single test name"
    echo ""
}

VERBOSITY_FLAGS=
DRY_MODE_FLAG=
MACHINE_UNDER_TEST=
SYSROOT_INSPECT=

while getopts "hdvkulm:s" option; do
  case $option in
    h)
      usage
      exit 0
      ;;
    d)
      DRY_MODE_FLAG="-d"
      ;;
    v)
      VERBOSITY_FLAGS=" $VERBOSITY_FLAGS -v"
      ;;
    k)
      CONTINUE_ON_FAILURE=1
      ;;
    u)
      MOVE_TEST_OUTPUT=1
      ;;
    l)
      find . -type d -name test_\* | cut -c 3-
      exit 0
      ;;
    m)
      MACHINE_UNDER_TEST=$OPTARG
      ;;
    s)
      SYSROOT_INSPECT=1
      ;;
    \?)
      usage
      exit 1
      ;;
  esac
done

shift $(($OPTIND - 1))

if [ "$#" -eq 0 ]; then
    TESTS=$(find . -type d -name test_\* | sort)
else
    TESTS="$@"
fi

if [ -n "$SYSROOT_INSPECT" ] && [ $# -ne 1 ] ; then
    usage
    exit 1
fi

TMP_DIR="$(mktemp -d)"

echo "Building source..."
( cd .. &&
  autoreconf -i &&
  ./configure \
    --prefix=/\
    --exec_prefix=/usr\
    --datarootdir=/usr/share ) > "$TMP_DIR/build_output" 2>&1
if [ $? -ne 0 ]; then
    cat "$TMP_DIR/build_output"
    exit 1
fi

install_sysroot()
{
    cp -a --dereference "$1"/* "$2"
}

remove_sysroot()
{
    SRC=$1
    find "$1" -empty -type d | sed -e "s%$SRC/%%g" |
    while read dir_name; do rm -R "$TMP_DIR/sand box/$dir_name"; done
    find "$1" -type f | sed -e "s%$SRC/%%g" |
    while read file_name; do rm "$TMP_DIR/sand box/$file_name"; done
}

populate_sysroot()
{
    TEST="$1"
    MACHINE="$2"

    mkdir -p "$TMP_DIR/server"
    mkdir -p "$TMP_DIR/sand box"
    mkdir -p "$TMP_DIR/sand box/tmp"
    mkdir -p "$TMP_DIR/sand box/mnt"

    ( cd .. && make install DESTDIR="$TMP_DIR/sand box/"
    ) > "$TMP_DIR/install_log" 2>&1
    if [ $? -ne 0 ]; then
        echo "make install failed"
        cat "$TMP_DIR/install_log"
    fi

    install_sysroot server "$TMP_DIR/server/"
    install_sysroot skeleton "$TMP_DIR/sand box/"
    install_sysroot skeleton_${MACHINE} "$TMP_DIR/sand box/"
    if [ -d "$TEST"/missing_files ]; then
        remove_sysroot "$TEST"/missing_files
    fi
    if [ -d "$TEST"/missing_files_${MACHINE} ]; then
        remove_sysroot "$TEST"/missing_files
    fi
    if [ -d "$TEST"/files ]; then
        install_sysroot "$TEST"/files "$TMP_DIR/sand box/"
    fi
    if [ -d "$TEST"/files_${MACHINE} ]; then
        install_sysroot "$TEST"/files_${MACHINE} "$TMP_DIR/sand box/"
    fi

    # gpg: WARNING: unsafe permissions on homedir '..'
    chmod 0700 "$TMP_DIR/sand box/etc/gnupg"
}

if [ -n "$SYSROOT_INSPECT" ]; then
    populate_sysroot "$TESTS" "$DEFAULT_MACHINE"
    echo ""
    echo "sysroot: \"$TMP_DIR/sand box\" remove after use"
    echo ""
    if type mc > /dev/null 2>&1; then
        gnome-terminal -e "mc \"$TMP_DIR/sand box\" $TESTS/files"
    fi
    exit 0
fi

TST_OUTPUT="$TMP_DIR/output"
RESCUE_ROOT_DIR="sand box"
WGET_SETUP_TIME=1
WGET_SIGNATURE_BACKOFF_TIME=0
IFUP_MAX_ATTEMPTS=3
IFUP_ATTEMPT_DELAY=0
FAILURE_SLEEP_TIME=0
export RESCUE_ROOT_DIR
export WGET_SETUP_TIME
export WGET_SIGNATURE_BACKOFF_TIME
export IFUP_MAX_ATTEMPTS
export IFUP_ATTEMPT_DELAY
export FAILURE_SLEEP_TIME
echo "Test dir: $TMP_DIR"
PATH="$TMP_DIR/sand box/sbin:$TMP_DIR/sand box/bin:$TMP_DIR/sand box/usr/bin:$PATH"
export PATH
for TEST in $TESTS; do

    MACHINES="$DEFAULT_MACHINE"
    if [ -e "$TEST"/test.conf ]; then
        parse_configuration "$TEST"/test.conf\
            MACHINES
    fi

    for MACHINE in $MACHINES; do
        if [ -n "$MACHINE_UNDER_TEST" ] &&
            [ "$MACHINE" != "$MACHINE_UNDER_TEST" ]; then
            continue
        fi

        DESC="${TEST#./}(${MACHINE})"
        printf "%-58s" "$DESC"

        populate_sysroot "$TEST" $MACHINE

        ( cd "$TMP_DIR" &&
          busybox sh "$(find . -name dss-boot)" $VERBOSITY_FLAGS $DRY_MODE_FLAG
        ) > "$TST_OUTPUT" 2>&1 < /dev/null

        EXPECTED="${TEST}"/expected_${MACHINE}
        if [ ! -r "${EXPECTED}" ]; then
            EXPECTED="${TEST}"/expected
        fi

        DIFF_OUTPUT="$(diff --unidirectional-new-file "$EXPECTED" "$TST_OUTPUT" 2>&1)"
        DIFF_STATUS="$?"
        if [ "$DIFF_STATUS" -ne 0 ] && [ -n "$MOVE_TEST_OUTPUT" ]; then
            mv "$TST_OUTPUT" "$EXPECTED"
        fi
        rm -Rf "$TMP_DIR/sand box"
        if [ "$DIFF_STATUS" -eq 0 ]; then
            echo " passed."
        else
            echo " failed!"
            if [ -n "$MOVE_TEST_OUTPUT" ]; then
                echo " Updated expected file with the new output."
            fi
            echo "Output of diff $EXPECTED $TST_OUTPUT"
            echo "$DIFF_OUTPUT"
            if [ -z "$CONTINUE_ON_FAILURE" ]; then
                ( cd .. && make distclean )  >> "$TMP_DIR/build_output" 2>&1
                exit 1
            fi
        fi
        # cleanup if next text uses a different machine
        ( cd .. && MACHINE=$MACHINE make clean  >> "$TMP_DIR/build_output" 2>&1)
    done
done

( cd .. && make distclean ) >> "$TMP_DIR/build_output" 2>&1
rm -Rf "$TMP_DIR"

